//
//  DrawingView.m
//  Drawing
//
//  Created by Maneesh Goel on 8/21/12.
//  Copyright (c) 2012 Maneesh Goel. All rights reserved.
//

#import "DrawingView.h"
@interface DrawingView() {
    CGContextRef context;
}
@end
@implementation DrawingView
@synthesize lines = _lines;
@synthesize circles = _circles;
@synthesize triangles = _triangles;
@synthesize freeLines = _freeLines;

@synthesize currentLine = _currentLine;
@synthesize currentCircle = _currentCircle;
@synthesize currentTriangle = _currentTriangle;
@synthesize currentFreeLine = _currentFreeLine;

@synthesize currentDrawMode = _currentDrawMode;

-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    NSLog(@"touch began");
    if (self.currentDrawMode == @"Line") {
        self.currentLine = [Line new];
        self.currentLine.startPoint = [[touches anyObject] locationInView:self];
    } else if (self.currentDrawMode == @"Circle") {
        self.currentCircle = [Circle new];
        self.currentCircle.circleCenter = [[touches anyObject] locationInView:self];
    } else if (self.currentDrawMode == @"Triangle") {
        //FILL IN HERE
        if (!self.currentTriangle) {
            self.currentTriangle = [Triangle new];
        }
        for (UITouch *touch in [touches allObjects]) {
            [self.currentTriangle.points setObject:[NSValue valueWithCGPoint:[touch locationInView:self]] forKey:[NSValue valueWithNonretainedObject:touch]];
        }
    } else if (self.currentDrawMode == @"Free") {
        self.currentFreeLine = [FreeLine new];
        self.currentFreeLine.startPoint = [[touches anyObject] locationInView:self];
    }
}

-(void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
    NSLog(@"touch moved");
    if (self.currentDrawMode == @"Line") {
        self.currentLine.endPoint = [[touches anyObject] locationInView:self];
    } else if (self.currentDrawMode == @"Circle") {
        self.currentCircle.radiusPoint = [[touches anyObject] locationInView:self];
    } else if (self.currentDrawMode == @"Triangle") {
        // FILL IN HERE
        if (self.currentTriangle.points.count == 3) {
            for (UITouch *touch in [touches allObjects]) {
                [self.currentTriangle.points setObject:[NSValue valueWithCGPoint:[touch locationInView:self]] forKey:[NSValue valueWithNonretainedObject:touch]];
            }
        }
    } else if (self.currentDrawMode == @"Free") {
        self.currentFreeLine.endPoint = [[touches anyObject] locationInView:self];
        [self.freeLines addObject:self.currentFreeLine];
        self.currentFreeLine = [FreeLine new];
        self.currentFreeLine.startPoint = [[touches anyObject] locationInView:self];
        self.currentFreeLine.endPoint = [[touches anyObject] locationInView:self];
    }
    [self setNeedsDisplay];
}

-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    NSLog(@"touch ended");
    if (self.currentDrawMode == @"Line") {
        self.currentLine.endPoint = [[touches anyObject] locationInView:self];
        [self.lines addObject:self.currentLine];
        self.currentLine = nil;
    } else if (self.currentDrawMode == @"Circle") {
        self.currentCircle.radiusPoint = [[touches anyObject] locationInView:self];
        [self.circles addObject:self.currentCircle];
        self.currentCircle = nil;
    } else if (self.currentDrawMode == @"Triangle") {
        // FILL IN HERE
        if (self.currentTriangle.points.count == 3) {
            [self.triangles addObject:self.currentTriangle];
        }
        self.currentTriangle = nil;
    } else if (self.currentDrawMode == @"Free") {
        self.currentFreeLine.endPoint = [[touches anyObject] locationInView:self];
        [self.freeLines addObject:self.currentFreeLine];
        self.currentFreeLine = nil;
    }
    [self setNeedsDisplay];
}

- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        _lines = [[NSMutableArray alloc] init];
        _circles = [[NSMutableArray alloc] init];
        _triangles = [[NSMutableArray alloc] init];
        _freeLines = [[NSMutableArray alloc] init];
        context = UIGraphicsGetCurrentContext();
        self.backgroundColor = [UIColor whiteColor];
        _currentDrawMode = @"Line";
        
        UIButton *lineButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
        lineButton.frame = CGRectMake(25.0, 410.0, 45.0, 30.0);
        [lineButton addTarget:self action:@selector(addLine) forControlEvents:UIControlEventTouchUpInside];
        [lineButton setTitle:@"Line!" forState:UIControlStateNormal];
        
        UIButton *circleButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
        circleButton.frame = CGRectMake(80, 410.0, 60.0, 30.0);
        [circleButton addTarget:self action:@selector(addCircle) forControlEvents:UIControlEventTouchUpInside];
        [circleButton setTitle:@"Circle!" forState:UIControlStateNormal];
        
        UIButton *freeFormButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
        freeFormButton.frame = CGRectMake(150.0, 410.0, 60.0, 30.0);
        [freeFormButton addTarget:self action:@selector(addFree) forControlEvents:UIControlEventTouchUpInside];
        [freeFormButton setTitle:@"Free!" forState:UIControlStateNormal];
        
        UIButton *triangleButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
        triangleButton.frame = CGRectMake(220.0, 410.0, 75.0, 30.0);
        [triangleButton addTarget:self action:@selector(addTriangle) forControlEvents:UIControlEventTouchUpInside];
        [triangleButton setTitle:@"Triangle!" forState:UIControlStateNormal];
        
        UIButton *clearButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
        clearButton.frame = CGRectMake(215.0, 20.0, 75.0, 30.0);
        [clearButton addTarget:self action:@selector(clearAll) forControlEvents:UIControlEventTouchUpInside];
        [clearButton setTitle:@"Clear!" forState:UIControlStateNormal];
        
        [self addSubview:lineButton];
        [self addSubview:circleButton];
        [self addSubview:freeFormButton];
        [self addSubview:triangleButton];
        [self addSubview:clearButton];
    }
    return self;
}

-(void)addLine {
    self.currentDrawMode = @"Line";
    self.multipleTouchEnabled = NO;
}

-(void)addCircle {
    self.currentDrawMode = @"Circle";
    self.multipleTouchEnabled = NO;
}

-(void)addFree {
    self.currentDrawMode = @"Free";
    self.multipleTouchEnabled = NO;
}

-(void)addTriangle {
    self.currentTriangle =nil;
    self.currentDrawMode = @"Triangle";
    self.multipleTouchEnabled = YES;
}

-(void)clearAll {
    [self.lines removeAllObjects];
    [self.circles removeAllObjects];
    [self.triangles removeAllObjects];
    [self.freeLines removeAllObjects];
    [self setNeedsDisplay];
}


- (void)drawRect:(CGRect)rect
{
    // Drawing code
    context = UIGraphicsGetCurrentContext();
    [[UIColor blackColor] set];
    [self.currentLine drawLine:context];
    for(Line *line in self.lines) {
        [line drawLine:context];
    }
    
    [self.currentCircle drawCircle:context];
    for(Circle *circle in self.circles) {
        [circle drawCircle:context];
    }
    
    if (self.currentTriangle.points.count == 3) {
        [self.currentTriangle drawTriangle:context];
    }
    for(Triangle *triangle in self.triangles) {
        [triangle drawTriangle:context];
    }
    
    [self.currentFreeLine drawFreeLine:context];
    for(FreeLine *freeLine in self.freeLines) {
        [freeLine drawFreeLine:context];
    }
    
}

@end
